@@ -123,23 +123,12 @@ dist_prob_to_class <- function(geno, prob.thres = 0.9) {
 }
 
 #' Export data to \code{polymapR}
+#' 
+#' See examples at \url{https://rpubs.com/mmollin/tetra_mappoly_vignette}.
+#' 
 #' @param data.in an object of class \code{mappoly.data}
 #' @return a dosage \code{matrix} 
 #' @author Marcelo Mollinari, \email{mmollin@ncsu.edu}
-#' @examples
-#' \donttest{
-#' require(polymapR)
-#' dat<-export_data_to_polymapR(hexafake)
-#' F1checked <- checkF1(dosage_matrix = dat, 
-#'                      parent1 = "P1",
-#'                      parent2 = "P2",
-#'                      F1 = colnames(dat)[-c(1:2)],
-#'                      polysomic = TRUE, 
-#'                      disomic = FALSE, 
-#'                      mixed = FALSE, 
-#'                      ploidy = 6)
-#'  head(F1checked$checked_F1)
-#'}  
 #' @export export_data_to_polymapR
 export_data_to_polymapR <- function(data.in)
 {
@@ -244,6 +233,51 @@ compare_haplotypes <- function(m, h1, h2) {
   list(is.same.haplo = !any(is.na(haplo.ord)), haplo.ord = haplo.ord)
 }
 
+#' Genotypic information content 
+#' 
+#' This function plots the genotypic information content given 
+#' an object of class \code{mappoly.homoprob}.
+#' 
+#' @param hprobs an object of class \code{mappoly.homoprob}
+#' 
+#' @param P a string containing the name of parent P
+#' 
+#' @param Q a string containing the name of parent Q
+#' 
+
+#' 
+#' @examples
+#' \donttest{
+#'      w <- lapply(solcap.err.map[1:3], calc_genoprob)
+#'      h.prob <- calc_homoprob(w)
+#'      plot_GIC(h.prob)
+#' }
+#' 
+#' @importFrom dplyr mutate 
+#' @importFrom ggplot2 facet_wrap element_text ylim scale_color_discrete
+
+#' @export plot_GIC 
+plot_GIC <- function(hprobs, P = "P1", Q = "P2"){
+  if(!inherits(hprobs, "mappoly.homoprob"))
+    stop(" 'hprobs' should be of class 'mappoly.homoprob'")
+  LG <- map.position <- GIC <- p1 <- m1 <- homolog <- marker <- probability <- NULL
+  DF <- hprobs$homoprob %>% 
+    dplyr::mutate(p1 = probability * (1-probability)) %>%
+    group_by(marker, homolog, map.position, LG) %>%
+    summarise(m1 = sum(p1)) %>%
+    mutate(GIC = 1-(4/hprobs$info$nind)*m1 , parent = ifelse(homolog%in%letters[1:hprobs$info$m], P, Q))
+  head(as.data.frame(DF))
+  
+ print(ggplot(DF, aes(map.position, GIC, colour = homolog)) +
+    geom_smooth(alpha = .8, se = FALSE) + facet_wrap(~LG, nrow = 3, ncol = 5) +
+    facet_grid(parent~LG, scales = "free_x", space = "free_x") +
+    geom_hline(yintercept = .8, linetype="dashed") + ylim(0,1) +
+    theme(axis.text.x = element_text(angle = 90, hjust = 1)) +
+    scale_color_discrete(name = "Homologs") +
+    ylab("Genotypic Information Content") + xlab("Distance (cM)"))
+  return(invisible(DF))
+}
+
 #' Plot two overlapped haplotypes
 #'
 #' @param void internal function to be documented
@@ -377,10 +411,13 @@ perm_pars <- function(v) {
 #' @param void internal function to be documented
 #' @keywords internal
 #' @export
-#' @importFrom grDevices hcl
+#' @importFrom grDevices hcl col2rgb hsv rgb2hsv
 gg_color_hue <- function(n) {
-  hues = seq(15, 375, length = n + 1)
-  hcl(h = hues, l = 65, c = 100)[1:n]
+  x<-rgb2hsv(col2rgb("steelblue"))[, 1]
+  cols = seq(x[1], x[1] + 1, by = 1/n)
+  cols = cols[1:n]
+  cols[cols > 1] <- cols[cols > 1] - 1
+  return(hsv(cols, x[2], x[3]))
 }
 
 #' Update missing information
@@ -438,12 +475,16 @@ mrk_chisq_test<-function(x, m){
 #' @param input.seq a sequence object of class \code{mappoly.sequence}
 #' @param verbose if \code{TRUE} (default), the current progress is shown; if
 #'     \code{FALSE}, no output is produced
+#' @param x 	an object of the class mappoly.geno.ord
+#' @param ... 	currently ignored   
+#'   
 #' @author Marcelo Mollinari, \email{mmollin@ncsu.edu}
 #' @examples
-#'  s1<-make_seq_mappoly(tetra.solcap, "all")
-#'  o1<-get_genomic_order(s1)
-#'  head(o1)
-#' @export
+#' s1<-make_seq_mappoly(tetra.solcap, "all")
+#' o1<-get_genomic_order(s1)
+#' plot(o1)
+#' s.geno.ord <- make_seq_mappoly(o1)
+#' @export get_genomic_order
 get_genomic_order<-function(input.seq, verbose = TRUE){
   if (!inherits(input.seq, "mappoly.sequence")) {
     stop(deparse(substitute(input.seq)), 
@@ -454,23 +495,43 @@ get_genomic_order<-function(input.seq, verbose = TRUE){
       stop("No sequence or sequence position information found.")
     else{
       if (verbose) message("Ordering markers based on sequence information")
-      M<-data.frame(seq = input.seq$sequence, row.names = input.seq$seq.mrk.names)
-      return(M[order(M[,1]),])
+      M <- data.frame(seq = input.seq$sequence, row.names = input.seq$seq.mrk.names)
+      M.out <- M[order(M[,1]),]
     }
   } else if(all(is.na(input.seq$sequence))){
     if(all(is.na(input.seq$sequence.pos))) 
       stop("No sequence or sequence position information found.")
     else{
       if (verbose) message("Ordering markers based on sequence position information")
-      M<-data.frame(seq.pos = input.seq$sequence.pos, row.names = input.seq$seq.mrk.names)
-      return(M[order(as.numeric(M[,1])),])
+      M <- data.frame(seq.pos = input.seq$sequence.pos, row.names = input.seq$seq.mrk.names)
+      M.out <- M[order(as.numeric(M[,1])),]
     }
   } else{
     M<-data.frame(seq = input.seq$sequence, 
                   seq.pos = input.seq$sequence.pos, 
                   row.names = input.seq$seq.mrk.names)
-    return(M[order(as.numeric(M[,1]), as.numeric(M[,2])),])
+    M.out <- M[order(as.numeric(M[,1]), as.numeric(M[,2])),]
   }
+  structure(list(data.name = input.seq$data.name, ord = M.out), class = "mappoly.geno.ord")
+}
+
+#' @rdname get_genomic_order
+#' @export
+print.mappoly.geno.ord <- function(x, ...){
+  print(head(x$ord))
+}
+
+#' @rdname get_genomic_order
+#' @export
+#' @importFrom ggplot2 ggplot aes geom_point xlab ylab theme
+plot.mappoly.geno.ord <- function(x, ...){
+  seq <- seq.pos <- NULL
+  ggplot2::ggplot(as.data.frame(x$ord), 
+                  ggplot2::aes(x=seq.pos, y=seq, group=as.factor(seq))) +
+    ggplot2::geom_point(ggplot2::aes(color=as.factor(seq)), shape = 108) +
+    ggplot2::xlab("position") + 
+    ggplot2::ylab("sequence") + 
+    ggplot2::theme(legend.position = "none") 
 }
 
 #' Remove markers from a map
@@ -967,7 +1028,7 @@ add_marker <- function(input.map,  mrk, pos, rf.matrix, genoprob = NULL,
 #'     analysis and haplotype phasing in experimental autopolyploid
 #'     populations with high ploidy level using hidden Markov
 #'     models, _G3: Genes, Genomes, Genetics_. 
-#'     \url{https://doi.org/10.1534/g3.119.400378}
+#'     \doi{10.1534/g3.119.400378}
 #'     
 #' @export check_data_sanity
 check_data_sanity<-function(x){
@@ -1168,7 +1229,7 @@ check_data_dist_sanity <- function(x){
 #'     analysis and haplotype phasing in experimental autopolyploid
 #'     populations with high ploidy level using hidden Markov
 #'     models, _G3: Genes, Genomes, Genetics_. 
-#'     \url{https://doi.org/10.1534/g3.119.400378} 
+#'     \doi{10.1534/g3.119.400378} 
 #'
 #' @export merge_datasets
 #' @importFrom dplyr bind_rows arrange
@@ -1306,7 +1367,7 @@ summary_maps = function(map.list, verbose = TRUE){
          " is not a list containing 'mappoly.map' objects.")
   results = data.frame("LG" = as.character(seq(1,length(map.list),1)),
                        "Genomic sequence" = as.character(unlist(lapply(map.list, function(x) paste(unique(x$info$sequence), collapse = "-")))),
-                       "Map size (cM)" = unlist(lapply(map.list, function(x) round(sum(c(0, imf_h(x$maps[[1]]$seq.rf))), 2))),
+                       "Map length (cM)" = unlist(lapply(map.list, function(x) round(sum(c(0, imf_h(x$maps[[1]]$seq.rf))), 2))),
                        "Markers/cM" = round(unlist(lapply(map.list, function(x) x$info$n.mrk/(round(sum(c(0, imf_h(x$maps[[1]]$seq.rf))), 2)))),2),
                        "Simplex" = unlist(lapply(map.list, function(x) sum(get_tab_mrks(x)[rbind(c(1,2),c(2,1),c(x$info$m,(x$info$m+1)),c((x$info$m+1),x$info$m))]))),
                        "Double-simplex" = unlist(lapply(map.list, function(x) sum(get_tab_mrks(x)[rbind(c(2,2),c(x$info$m,x$info$m))]))),
@@ -1314,7 +1375,7 @@ summary_maps = function(map.list, verbose = TRUE){
                        "Total" = unlist(lapply(map.list, function(x) x$info$n.mrk)),
                        "Max gap" = unlist(lapply(map.list, function(x) round(imf_h(max(x$maps[[1]]$seq.rf)),2))),
                        check.names = FALSE, stringsAsFactors = F)
-  results = rbind(results, c('Total', NA, sum(as.numeric(results$`Map size (cM)`)), round(mean(as.numeric(results$`Markers/cM`)),2), sum(as.numeric(results$Simplex)), sum(as.numeric(results$`Double-simplex`)), sum(as.numeric(results$Multiplex)), sum(as.numeric(results$Total)), round(mean(as.numeric(results$`Max gap`)),2)))
+  results = rbind(results, c('Total', NA, sum(as.numeric(results$`Map length (cM)`)), round(mean(as.numeric(results$`Markers/cM`)),2), sum(as.numeric(results$Simplex)), sum(as.numeric(results$`Double-simplex`)), sum(as.numeric(results$Multiplex)), sum(as.numeric(results$Total)), round(mean(as.numeric(results$`Max gap`)),2)))
   if (verbose){
     all.mrks = unlist(lapply(map.list, function(x) return(x$info$mrk.names)))
     if (!any(get(map.list[[1]]$info$data.name, pos = 1)$elim.correspondence$elim %in% all.mrks))
