@@ -12,8 +12,8 @@ test_that("test several utility functions", {
   tpt<-est_pairwise_rf(make_seq_mappoly(tetra.solcap, 1:30))
   x1<-make_seq_mappoly(tetra.solcap, 1:20)
   x2<-make_seq_mappoly(tetra.solcap, 21:40)
-  x1<-as.numeric(mappoly:::pos_twopt_est(x1))
-  x2<-as.numeric(mappoly:::pos_twopt_est(x2))
+  x1<-as.numeric(pos_twopt_est(x1))
+  x2<-as.numeric(pos_twopt_est(x2))
   expect_equal(as.numeric(crossprod(x1,x2)), 1)
   M<-rf_list_to_matrix(tpt, shared.alleles = TRUE)
   expect_equal(round(sum(get_rf_from_mat(M$rec.mat), na.rm = TRUE), 6), 3.913633)
@@ -27,12 +27,12 @@ test_that("test several utility functions", {
   plot(a2)
   a3<-dist_prob_to_class(a2$geno)
   expect_equal(prod(dim(a3$geno.dose)), nrow(a3$geno))
-  w1<-export_data_to_polymapR(hexafake)
-  expect_equal(dim(w1) , c(1500, 302))
-  expect_equal(sum(w1), 461530)
-  w2<-export_data_to_polymapR(tetra.solcap)
-  expect_equal(dim(w2) , c(3679, 162))
-  expect_equal(sum(w2, na.rm = TRUE), 1058080)
+  #w1<-export_data_to_polymapR(hexafake)
+  #expect_equal(dim(w1) , c(1500, 302))
+  #expect_equal(sum(w1), 461530)
+  #w2<-export_data_to_polymapR(tetra.solcap)
+  #expect_equal(dim(w2) , c(3679, 162))
+  #expect_equal(sum(w2, na.rm = TRUE), 1058080)
   expect_equal(round(mf_h(20),6), 0.16484)
   expect_equal(round(mf_k(20),6), 0.189974)
   expect_equal(round(mf_m(20),6), 0.2)
@@ -51,9 +51,10 @@ test_that("test several utility functions", {
   a<-sample_data(a, n=30)
   w<-update_missing(a, prob.thres = .7)
   expect_is(w, "mappoly.data")
-  w2<-get_genomic_order(make_seq_mappoly(hexafake, "all"))
-  w3<-round(as.numeric(crossprod(w2$seq.pos)/10e17),3)
-  expect_equal(w3, 0.495)
+  w2 <- get_genomic_order(make_seq_mappoly(hexafake, "all"))
+  w3 <- as.numeric(crossprod(w2$ord$seq.pos)/10e17)
+  expect_is(plot(w2), "ggplot")
+  expect_equal(w3, 0.4952, tolerance = 1e-3)
   ##test drop
   w4<-get_submap(solcap.dose.map[[1]], 1:10, reestimate.rf = FALSE)
   s4<-make_seq_mappoly(w4)
